@@ -19,7 +19,10 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
-#include "../config.h"
+#if HAVE_CONFIG_H
+# include "../config.h"
+#endif
+
 #include "common.h"
 #include "hacks.h"
 #include "command.h"
@@ -304,10 +307,10 @@ _partition_warn_busy (PedPartition* part)
 			_("Partition %s is being used.  You must unmount it "
 			  "before you modify it with Parted."),
 			path);
-		ped_free (path);
+		PED_FREE (path);
 		return 0;
 	}
-	ped_free (path);
+	PED_FREE (path);
 	return 1;
 }
 
@@ -679,7 +682,7 @@ do_mkpart (PedDisk **disk)
 			return 0;
 	}
 	if (peek_word)
-		ped_free (peek_word);
+		PED_FREE (peek_word);
 
 
 	pos.start.sector = 0LL;
@@ -802,8 +805,8 @@ print_sector_compact_and_percent (PedSector sector, PedDevice* dev)
 
 	printf ("%s (%s)\n", compact, percent);
 
-	ped_free (compact);
-	ped_free (percent);
+	PED_FREE (compact);
+	PED_FREE (percent);
 }
 
 static int
@@ -1116,7 +1119,7 @@ do_print (PedDisk **disk)
 		if (compat_mode)
 			check_partition_consistency(part);
 	}
-	ped_free (cyl_size);
+	PED_FREE (cyl_size);
 
 	/*if (fdisk_list_table == 1)
 	  do_quit (disk);*/
@@ -1161,12 +1164,12 @@ part_xprint (PedDisk **disk, int extend)
 		part_chs = ped_unit_format_custom(dev, part->geom.start,
 	                                          PED_UNIT_CHS);
 		sscanf(part_chs, "%d,%d,%d", &start_cyl, &start_head, &start_sector);
-		ped_free(part_chs);
+		PED_FREE (part_chs);
 
 		part_chs = ped_unit_format_custom(dev, part->geom.end,
 	                                          PED_UNIT_CHS);
 		sscanf(part_chs, "%d,%d,%d", &end_cyl, &end_head, &end_sector);
-		ped_free(part_chs);
+		PED_FREE (part_chs);
 
 
 		is_boot = ped_partition_get_flag(part,PED_PARTITION_BOOT);
@@ -1384,8 +1387,8 @@ _rescue_add_partition (PedPartition* part)
 		fs_type->name, ped_partition_type_get_name (part->type),
 		found_start, found_end);
 	ped_geometry_destroy (probed);
-	ped_free (found_start);
-	ped_free (found_end);
+	PED_FREE (found_start);
+	PED_FREE (found_end);
 
 	switch (ex_opt) {
 		case PED_EXCEPTION_CANCEL: return -1;
