@@ -28,7 +28,10 @@
 #include <unistd.h>
 #include <setjmp.h>
 
-#include "config.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "command.h"
 #include "strlist.h"
 #include "ui.h"
@@ -51,7 +54,7 @@ extern int fdisk_list_table;
 #define _(String) dgettext (PACKAGE, String)
 
 /* BEGIN readline stuff */
-#if HAVE_READLINE
+#if HAVE_LIBREADLINE
 
 /* Termcap need this buffer to initialize terminal database, I think
    the db is stored into this buffer but I'm not sure. */
@@ -66,7 +69,9 @@ static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 # if HAVE_TERMCAP_H
 #  include <termcap.h>
 # else
+#  if HAVE_LIBTERMCAP
     extern int tgetnum (char* key);
+#  endif
 # endif /* HAVE_TERMCAP_H */
 
 /* readline/readline.h */
@@ -93,7 +98,7 @@ static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 #  define rl_compentry_func_t void
 # endif
 
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE */
 
 /* END readline stuff */
 
@@ -127,7 +132,7 @@ static PedExceptionOption current_exception_opt = 0;
 static prompt_possibilities = 1;
 
 
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE && HAVE_LIBTERMCAP
 /* This function initialize the termcap library */
 void init_termcap_library(void)
 {
@@ -189,7 +194,6 @@ fdisk_complete_function (char* text, int start, int end)
   return rl_completion_matches (text, (rl_compentry_func_t*) command_generator);
 }
 
-#if HAVE_READLINE_HISTORY_H
 static void
 _add_history_unique (const char* line)
 {
@@ -215,9 +219,8 @@ _add_history_unique (const char* line)
   /* no match */
   add_history(line);
 }
-#endif /* HAVE_READLINE_HISTORY_H */
 
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE && HAVE_LIBTERMCAP_H */
 
 static void
 interrupt_handler (int signum)
@@ -259,7 +262,7 @@ _readline (const char* prompt, const StrList* possibilities)
   fdisk_wipe_line ();
 
   if (!fdisk_opt_script_mode)
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
     { /* !fdisk_opt_script_mode && HAVE_READLINE */
       line = readline(prompt);
       if(line && *line)
@@ -275,6 +278,12 @@ _readline (const char* prompt, const StrList* possibilities)
 	  free (line);
 	  line = NULL;
 	}
+      else
+	{
+	  char *end = strchr(line, 0x0a);
+	  if(end)
+	    *end = 0;
+	}
     }
 #endif
   /* MMM... I assume that `disk_opt_script_mode' is true when fdisk
@@ -289,6 +298,12 @@ _readline (const char* prompt, const StrList* possibilities)
 	  free (line);
 	  line = NULL;
 	}
+      else
+	{
+	  char *end = strchr(line, 0x0a);
+	  if(end)
+	    *end = 0;
+	}
     }
   readline_state.in_readline = 0;
   return line;
@@ -717,8 +732,9 @@ fdisk_interactive_mode(PedDevice **dev, FdiskCommand* cmd_list[])
   
   /* List the specified disk. */
   if (fdisk_list_table)
-    fdisk_do_list_devices(&disk);
-  
+    /* http://lists.gnu.org/archive/html/bug-fdisk/2008-12/msg00011.html */
+    /* fdisk_do_list_devices(&disk); */
+    fdisk_do_list_devices(disk);
 
   fdisk_print_using_dev (disk->dev);
 
@@ -844,7 +860,7 @@ fdisk_screen_width ()
     return 32768;	/* no wrapping ;) */
 
 /* HACK: don't specify termcap separately - it'll annoy the users. */
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
   if(termcap_status == TERMCAP_OK)
     width = tgetnum ("co");
 #endif
@@ -1398,15 +1414,15 @@ fdisk_command_line_get_sector (const char* prompt, PedDevice* dev, PedSector* va
 	if (input && *value && !strcmp (input, def_str)) {
 		if (range) {
 			*range = ped_geometry_new (dev, *value, 1);
-			ped_free (def_str);
+			PED_FREE (def_str);
 			return *range != NULL;
 		}
 
-		ped_free (def_str);
+		PED_FREE (def_str);
 		return 1;
 	}
 
-	ped_free (def_str);
+	PED_FREE (def_str);
 	if (!input) {
 		*value = 0;
 		if (range)
@@ -1436,12 +1452,12 @@ fdisk_init_ui ()
     return 0;
   ped_exception_set_handler (exception_handler);
 
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
   rl_initialize ();
   rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
-  readline_state.in_readline = 0;
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE */
 
+  readline_state.in_readline = 0; 
   signal (SIGINT, &interrupt_handler);
 
   return 1;
@@ -1480,7 +1496,7 @@ fdisk_command_line_get_part_flag (const char* prompt, const PedPartition* part,
 
 	if (flag_name) {
 		*flag = ped_partition_flag_get_by_name (flag_name);
-		ped_free (flag_name);
+		PED_FREE (flag_name);
 		return 1;
 	} else {
 		return 0;
